/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.datamodel.xdb.embl;

import jalview.bin.Cache;
import jalview.datamodel.DBRefEntry;
import jalview.ws.dbsources.Uniprot;

import java.io.File;
import java.io.FileReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.net.URL;
import java.util.Vector;

import org.exolab.castor.mapping.Mapping;
import org.exolab.castor.xml.Unmarshaller;

/**
 * Data model for entries returned from an EMBL query, as marshalled by a Castor
 * binding file
 * 
 * For example: http://www.ebi.ac.uk/Tools/dbfetch/dbfetch/embl/x53828/emblxml
 * 
 * @see embl_mapping.xml
 */
public class EmblFile
{
  Vector<EmblEntry> entries;

  Vector<EmblError> errors;

  /**
   * @return the entries
   */
  public Vector<EmblEntry> getEntries()
  {
    return entries;
  }

  /**
   * @param entries
   *          the entries to set
   */
  public void setEntries(Vector<EmblEntry> entries)
  {
    this.entries = entries;
  }

  /**
   * @return the errors
   */
  public Vector<EmblError> getErrors()
  {
    return errors;
  }

  /**
   * @param errors
   *          the errors to set
   */
  public void setErrors(Vector<EmblError> errors)
  {
    this.errors = errors;
  }

  /**
   * Parse an Embl XML file into an EmblEntry object
   * 
   * @param file
   * @return parsed EmblXML or null if exceptions were raised
   */
  public static EmblEntry getEmblEntry(File file)
  {
    if (file == null)
    {
      return null;
    }
    try
    {
      return EmblFile.getEntry(new FileReader(file));
    } catch (Exception e)
    {
      System.err.println("Exception whilst reading EMBLfile from " + file);
      e.printStackTrace(System.err);
    }
    return null;
  }

  /**
   * Reads the XML response from file and unmarshals into a Java object
   * 
   * @param fileReader
   * @return
   */
  public static EmblEntry getEntry(Reader fileReader)
  {
    EmblEntry record = new EmblEntry();
    try
    {
      // 1. Load the mapping information from the file
      Mapping map = new Mapping(record.getClass().getClassLoader());

      URL url = record.getClass().getResource("/embl_mapping.xml");
      map.loadMapping(url);

      // 2. Unmarshal the data
      Unmarshaller unmar = new Unmarshaller(record);
      try
      {
        if (Cache.getDefault(Cache.CASTORLOGLEVEL,
                "debug").equalsIgnoreCase("DEBUG"))
        {
          unmar.setDebug(Cache.log.isDebugEnabled());
          // unmar.setDebug(true);// uncomment to debug unmarshalling
        }
      } catch (Exception e)
      {
      }
      unmar.setIgnoreExtraElements(true);
      unmar.setIgnoreExtraAttributes(true);
      unmar.setMapping(map);
      unmar.setLogWriter(new PrintWriter(System.out));
      record = (EmblEntry) unmar.unmarshal(fileReader);

      canonicaliseDbRefs(record);
    } catch (Exception e)
    {
      e.printStackTrace(System.err);
      record = null;
    }

    return record;
  }

  /**
   * Change blank version to "0" in any DBRefEntry, to ensure consistent
   * comparison with other DBRefEntry in Jalview
   * 
   * @param entry
   * @see Uniprot#getDbVersion
   */
  static void canonicaliseDbRefs(EmblEntry entry)
  {
    if (entry == null)
    {
      return;
    }
//    for (EmblEntry entry : record.getEntries())
//    {
      if (entry.getDbRefs() != null)
      {
        for (DBRefEntry dbref : entry.getDbRefs())
        {
          if ("".equals(dbref.getVersion()))
          {
            dbref.setVersion("0");
          }
        }
      }

    if (entry.getFeatures() != null)
      {
        for (EmblFeature feature : entry.getFeatures())
        {
          if (feature.getDbRefs() != null)
          {
            for (DBRefEntry dbref : feature.getDbRefs())
            {
              if ("".equals(dbref.getVersion()))
              {
                dbref.setVersion("0");
              }
            }
          }
        }
      }
    // }
  }
}
